
! Program DrawBlade Draws a skating blade runner using postscript.
! The intention was so I could try to make my own blades.
! See http://mgrunes.com/boots/Make-your-own-blades for more info.

! It is mostly for figure skating blades, but could be used for 
! hockey and speed skating blades. 
! 
! Written by Mitchell Grunes.
! This is a free format fortran 90 program.
! Cygwin or linux compiles this using
!  gfortran -Wall DrawBlade.f90 -o DrawBlade

! Cygwin produces executable DrawBlade.exe; Linux produces DrawBlade.
 
! Postscript language reference:
!  https://www.adobe.com/jp/print/postscript/pdfs/PLRM.pdf
! See also https://www.adobe.chttps://personal.math.ubc.ca/~cass/graphics/text/old.pdf/last/ch4.pdfom/jp/print/postscript/pdfs/PLRM.pdf
!  https://helpx.adobe.com/x-productkb/global/troubleshoot-postscript-errors.html
!  https://personal.math.ubc.ca/~cass/graphics/text/old.pdf/last/ch4.pdf
 
! Can convert postscript to PDF using https://cloudconvert.com/ps-to-pdf.
! Or https://online2pdf.com/convert-ps-to-pdf seems able to take larger size pages.

! Can possibly convert postscript to Cad format using a free Cad 
!  program, like DoubleCad XT - download from
!  https://www.turbocad.com/content/doublecad-xt-v5
!  Instructions are at
!  https://www.youtube.com/watch?v=RDd1M9Zvl44 
! Or FreeCAD.https://wiki.freecad.org/Import_Export 
!  Says it can import PDF.
! Or LibreCad? https://librecad.org
!  Doesn't appear able to import postscript or pdf.
! Or LinuxCAD? 
! And search google for "Free 2D CAD software like AutoCAD"
! Or maybe use http://www.calvina.de/pstoedit to convert postscript to DXF?
! https://sourceforge.net/projects/thandxflib/postdownload describes a fortran 
!  library to write DXF.
! https://sourceforge.net/projects/thandxflib/files/dxflib/dxflib-1.0.0.zip/download

! https://h30434.www3.hp.com/t5/LaserJet-Printing/print-postscript-file/td-p/5594534 
! has instructions on using lpr to print postscript files

! Perhaps I should use 
! https://opensource.adobe.com/dc-acrobat-sdk-docs/pdfstandards/PDF32000_2008.pdf
! to instead write PDF files?
! Or
! https://www.opendesign.com/files/guestdownloads/OpenDesign_Specification_for_.dwg_files.pdf
! to create a DWG file?

! not used yet:
!                 x1 y1 x2 y2 r arct    Draw tangent arc. (To create rounded corner.)

! For my definitions of technical terms (there is no standardization!), like 
! "rocker", "rocker segment", spin rocker, sweet spot, and toe pick tooth names, 
! see file index.html.

common xmin,ymin,xmax,ymax,x,y,WidLin,io,iExtra
character*80 ParmFile,PsFile            ! Name of BladeDraw & Postscript files.
 
print*,'Enter 1 to input from blade parameter file, 0 to make a new one:'
io=1
read*,io
if(io.eq.0)print*,'Blade parameter file (Enter blank for "DrawBlade.txt") to write:'
if(io.ne.0)print*,'Blade parameter file (Enter blank for "DrawBlade.txt") to read:'
read(*,'(a80)')ParmFile
if(ParmFile.eq.' ')ParmFile='DrawBlade.txt'
print*,'ParmFile=',ParmFile
if(io.eq.0)open (1,file=ParmFile,status='new',err=1)
if(io.ne.0)open (1,file=ParmFile,status='old')
goto 2
1 print*,'File already exists - overwrite(0=no,1=yes?):'
  i=0
  read*,i
  if(i.eq.0)stop 'Cannot continue'
  if(io.eq.0)open (1,file=ParmFile,status='unknown')
 2 print*,'Output postscript file (Enter blank for "DrawBlade.ps"):'
if(io.eq.0)then
  read(*,'(a)')PsFile
  if(PsFile.eq.' ')PSFile='DrawBlade.ps'
  write(1,'(a)')PSFile
else
  read(1,'(a)')PSFile
  if(PsFile.eq.' ')PSFile='DrawBlade.ps'
endif

print*,'PsFile=',PSFile
open (2,file=PsFile,status='unknown')
write(2,'(a)')'%!PS'                    ! So print spools detect start of file
write(2,'(a)')'% ***Figure Skating Blade Profile created by Mitch Grunes''s DrawBlade.f90***'

call WriteBlank

call ReadInt('Add arc radius lines for illustrations(0=no? 1?)',iExtra,1)

xmin=1e20                               ! Initialize x,y min,max in inches.
ymin=1e20
xmax=-1e20
ymax=-1e20
 
print*,'Some MK blades have 1 main rocker and 1 spin rockers.'
print*,' (7 foot=84" main rocker, 9" spin rocker)'
print*,'Many JW blades have 1 main rocker and 2 spin rockers.'
print*,' (8 foot=96" main rocker, 27" (24"??) & 12" spin rockers)'
call ReadInt('# of rocker length segments(2? 3?)',nRocker,2)
 
write(2,'(a)')'/MyPath newpath {'
do iRocker=1,nRocker
  call WriteBlank
  print'(a,i0,a)','***Rocker #',iRocker,'***'
  if(iRocker.eq.1)then
    print'(a)','This is the main rocker segment!'
  elseif(iRocker.eq.2.and.nRocker.gt.2)then
    print'(a)','This is the backmost spin rocker segment!'
  elseif(iRocker.eq.nRocker.and.nRocker.eq.2)then
    print'(a)','This is the spin rocker segment!'
  elseif(iRocker.eq.nRocker.and.nRocker.gt.2)then
    print'(a)','This is the frontmost spin rocker segment!'
  else
    print'(a)','This is an intermediate spin rocker segment!'
  endif
 
  if(                 iRocker.eq.1)DefaultR=84
  if(                 iRocker.eq.2)DefaultR=9
  if(nRocker.eq.3.and.iRocker.eq.1)DefaultR=96
  if(nRocker.eq.3.and.iRocker.eq.2)DefaultR=27
  if(nRocker.eq.3.and.iRocker.eq.3)DefaultR=12
  call ReadFlt('Rocker radius (inches)',R,DefaultR)

  if(iRocker.eq.1)print'(a)',&
   & 'Rocker segment lengths are measured along curve, not straight!'
  call ReadFlt('Curved length of rocker segment (inches)',ArcLength,8.)
  DeltaTheta=ArcLength/R*180/3.1415926535897932 !Angle width from center, in degrees
  
! print*,'deltaTheta=',deltatheta     !##

  if(iRocker.eq.1)then
    Theta2=-90                          ! Make Place 1st "sweet spot" (intersection of 
                                        !  main rocker segment with first spin 
                                        !  rocker segment) hoizontal, by making the
                                        !  tangent line vertical.
    Theta1=Theta2-DeltaTheta            ! Radial angle at start of main rocker 
                                        ! segment.

    call ReadFlt('x value of leftmost point on blade(0?)',x,0.)

    x=x-R*cosd(theta1)                  ! 1st "sweet spot", by my definition 
                                        ! (the intersection of main rocker segment 
                                        ! with the first spin rocker segment).
    xCenter=x                           ! Center of main rocker arc.
    call ReadFlt('y value of lowest point on blade (0?)',y,0.)
    yCenter=y+R
!   print*,'deltaTheta=',deltatheta     !##
!   print*,'main rocker theta1=',theta1,'theta2=',theta2 !##
  else
    xCenter=x-R*cosd(theta2)
    yCenter=y-R*sind(theta2)
    Theta1=Theta2
    Theta2=Theta2+DeltaTheta
!   print*,'theta1=',theta1!##
!   print*,'theta2=',theta2!##
  endif                                 ! if(iRocker.eq.1)then
                                        ! Draw main rocker circular arc to path.
  print'(a,g0)','Direction angle of curve start=',Theta1+90
  print'(a,g0)','Direction angle of curve end=',Theta2+90  !##
  write(2,'(a,i0,a,g0,1x,g0,a,g0,a)') '% ***Rocker Segment ',iRocker, &
   & ' Center=',xCenter,yCenter,' inches Radius=',R,'***'
  write(2,'(a,g0,1x,g0,a)') '%***Angle range=',Theta1,Theta2,' deg***'
  write(2,'(a,g0,a)') '% ***(Curved length=',ArcLength,')***'
  call DrawArc(xCenter,yCenter,R,theta1,theta2)
  If(iRocker.eq.1)SaveHeight1=yCenter+R*sind(theta1)
enddo                                   ! do irocker=1,nrocker
                      
                  ! Draw toepick.
call WriteBlank
call ReadInt('# of toepick teeth',nPick,4)
do iPick=1,nPick
  call WriteBlank
  print'(a,i0,a)','***Toepick tooth# ',iPick,'***'
  write(2,'(a,i0,a)')'% ***ToePick #',iPick,' from back***'
  if(iPick.eq.1)print'(a)','This is the drag pick!'
  if(ipick.eq.nPick)print'(a)','This is the frontmost tooth!'
  if(iPick.eq.1)then
    print*,' '
    print'(a)','If the drag pick points forwards from the curve,'
    print'(a)','The following angle is >90 deg, else it''s <=90.'
    call ReadFlt('Angle from curve to drag pick(140?)',angle,140.)
!   print'(a,g0)','Direction angle of curve end=',Theta2+90                
    Theta2=angle+(Theta2+90-180)        ! Turn into angle to draw back edge 
                                        ! of drag pick from current x,y.
!   print*,'back edge direction=',Theta2
    DefaultTAng=70                      ! (Applies to other teeth)
    DefaultTLen=.25
  else                                  ! if(iPick.eq.1)then
    DefaultTAng=70
    call ReadFlt('Degrees from last tooth to this tooth',angle,DefaultTAng)
    Theta2=angle+(Theta2-180)           ! Turn into angle to draw back edge of 
                                        ! that tooth from current x,y
!   print*,'Theta2 for this tooth=',Theta2 !##
  endif                                 ! if(iPick.eq.1)then
                                        ! Draw back of that tooth.
  
  call ReadFlt('Length of back of that tooth',PickLen,DefaultTLen)
  write(2,'(a,g0,a,g0,a)') &
   & '% *Angle to back of tooth=',angle,' Length=',PickLen,'*'
  call LineTo(x+PickLen*cosd(Theta2),y+PickLen*sind(Theta2))
 
  call ReadFlt('Degrees in angle at bottom of tooth',angle,DefaultTAng)
  Theta2=(Theta2+180)-angle             ! Turn into angle to draw front edge
                                        ! of current tooth from current x,y
! print*,'Theta2 for front edge=',Theta2 !##
                                        
  call ReadFlt('Length of front of that tooth',PickLen,DefaultTLen)
                                        ! Draw front of the tooth
  write(2,'(a,g0,a,g0,a)') &
   & '% *Angle to front of tooth=',angle,' length ',PickLen,'*'
  call LineTo(x+PickLen*cosd(Theta2),y+PickLen*sind(Theta2))
enddo                                   ! do iPick=1,nPick 

print*,' '
if(io.eq.0)write(1,*)' '
call WriteBlank
call ReadFlt('Height of runner(0 to not draw top? 1?)',Height,1.)
if(Height.gt.0)then
  if(Height.lt.ymax)stop 'Height cannot be shorter than part of the blade!'
  xSave=x
  ySave=y
  write(2,'(a)')'% ***Draw top of blade runner***'
  call MoveTo(xmin,SaveHeight1)         ! Height is measured over back sweet spot
  call LineTo(xmin,SaveHeight1+Height)
  call LineTo(xSave,SaveHeight1+Height)
  call LineTo(xSave,ySave)
endif

call WriteBlank
write(2,'(a)')'% ***Diagram Setup***'
write(2,'(a)')'} def'

write(2,'(a)')'1 setlinecap'            ! Set shape of line ends for stroke
                                        ! (0=butt,1=round,2=square).
write(2,'(a)')'1 setlinejoin'           ! Set shape of corners for stroke
                                        ! (0=miter,1=round,2=bevel).

print*,'*** Setup Drawing Geometry ***'
print*,'Horizontal Blade length=',xmax-xmin

print'(a)','Add boundary lines & 1" tics to let you check'
print'(a)','printer scaling, and whether the blade fits on page!'

call ReadFlt('Scaled Tic Length (inches) (0=none; .2?)',TicLen,.2)

call ReadFlt('Paper margin size (inches) (.5?)',TheMargin,.5)

xmin2=xmin-2*TicLen                     ! Allow space for tics
ymin2=ymin-2*TicLen
xmax2=xmax+2*TicLen
ymax2=ymax+2*TicLen
if(TicLen.gt.0)ymax2=max(ymax2,ymin+1.001)! (I always draw at least 2 tics,
                                        ! so can check y scaling too.)
print'(a)','To print blade + Tics (if chosen) + margins at full'
print'(a,g0,a,g0,a)','scale takes ', &
 & xmax2-xmin2+TheMargin*2,' x ',ymax2-ymin2+TheMargin*2,' inches.'
print*,'Depending on paper size, that may fit with'
print*,'no rotation, 90 degree rotation, or, if you are desperate,'
print*,'diagonal rotation. Typical X x Y paper sizes are'
print*,'8.5"x11" (Letter), 8.5"*14" (Legal), 11"x17" (Tabloid),'
print*,'Ghostview supports all of those, e.g.,'
print*,' gv -media=Letter DrawBlade.ps'
print*,' gv -media=Legal DrawBlade.ps'
print*,' gv -media=Tabloid DrawBlade.ps'
print*,'(See https://www.gnu.org/software/gv/manual/gv.html for other size names)'
print*,' but many printers don''t.'
print*,' If you just want coordinates to guide a CAD'
print*,'program, just use pretend sizes that are a little larger'
print*,'than the blade.'

call ReadFlt('X-axis paper size(8.5?)',xPap,8.5)
call ReadFlt('Y-axis paper size(11?)' ,yPap,11.)
print*,'Somewhat longer blades fit with diagonal rotation.'
write(2,'(a,g0,a,g0,a)')'% *** This plot is designed for ',xPap,' x ',yPap,' Paper.***'
write(2,'(a,g0,1x,g0,a)')'<< /PageSize [',xPap*72,yPap*72,'>> setpagedevice'

call ReadInt('Rotation angle(0? 90? -1 to try diagonal fit)',irotate,90)

print*,'If you think the blade won''t fit at full scale, scale it by'
print*,'.5, and use a fancy photocopier to zoom it back out.'
call ReadFlt('Scale factor(1? .5?)',scale,1.)

write(2,'(a,g0,a)')'% Scale by ',scale,'*72 from Postscript 1/72 inch units.'
write(2,'(g0,1x,g0,a)')72*scale,72*scale,' scale'    

call ReadFlt('Line width (inches) (.01?)',WidLin,.01)
write(2,'(g0,a)')WidLin/scale,' setlinewidth'  ! Graphics line width in 1/72 inch (point) units.

call SetGeometry(TheMargin,xmin2,xmax2,ymin2,ymax2,xPap,yPap, &
 & irotate,scale)

print   '(2(a,g0,1x,g0))' ,&
 & '%Blade boundaries x: ',xmin,xmax,' y: ',ymin,ymax 
write(2,'(2(a,g0,1x,g0))') &
 & '%Blade boundaries x: ',xmin,xmax,' y: ',ymin,ymax 

write(2,'(a)')'MyPath stroke'
if(TicLen.ne.0)then
  call WriteBlank
  write(2,'(a)')'% ***X-axis & tic marks at scaled 1 inch intervals***'
                                        ! draw dropped x-axis & corners
  call DrawSeg(xmin,ymin-1.5*TicLen,xmax,ymin-1.5*TicLen)    
  call DrawSeg(xmin,ymin-1.5*TicLen,xmin,ymin- .5*TicLen)
  call DrawSeg(xmax,ymin-1.5*TicLen,xmax,ymin- .5*TicLen)

  !do xx=xmin,xmax+.0001                ! Draw x tics at scaled 1 inch intervals 
  do i=0,ifix(xmax-xmin+.0001)          ! Some may go beyond page boundaries.
    xx=xmin+i
    call DrawSeg(xx,ymin-2*TicLen,xx,ymin-1.5*TicLen)
  enddo

  write(2,'(a)')'% ***Y-axis & tic marks at scaled 1 inch intervals***'
                                        ! Draw left shifted y-axis & corners.
                                        ! Extend tic range to include more.
  call DrawSeg(xmin-1.5*TicLen,ymin,xmin-1.5*TicLen,ymax)    
  call DrawSeg(xmin-1.5*TicLen,ymin,xmin- .5*TicLen,ymin)
  call DrawSeg(xmin-1.5*Ticlen,ymax,xmin- .5*TicLen,ymax)
                                        ! Draw right shifted y-axis & corners
  call DrawSeg(xmax+1.5*TicLen,ymin,xmax+1.5*TicLen,ymax)   
  call DrawSeg(xmax+1.5*TicLen,ymin,xmax+ .5*TicLen,ymin)
  call DrawSeg(xmax+1.5*Ticlen,ymax,xmax+ .5*TicLen,ymax)

  !do yy=ymin,max(ymax,ymin+20)-ymin    ! Draw  y tics at scaled 1 inch intervals.
  do i=0,ifix(max(ymax,ymin+20)-ymin)   ! Some may go beyond page boundaries.
    yy=ymin+i
    call DrawSeg(xmin-2*TicLen,yy,xmin-1.5*TicLen,yy)
    call DrawSeg(xmax+2*TicLen,yy,xmax+1.5*TicLen,yy)
  enddo
endif
 
call WriteBlank
write(2,'(a)')'% ***Finished!***'
write(2,'(a)')'showpage'                ! prints the prepared page.
close(1)
close(2)
stop 'DrawBlade completed successfuly.'
end
! --------------------------------------------------------------------------------------
subroutine ReadInt(Prompt,num,iDefault) ! Read prompted integer, echo to parameter
                                        ! File, or to screen if from file.
common xmin,ymin,xmax,ymax,x,y,WidLin,io,iExtra
character*(*) Prompt
1 num=iDefault
if(io.eq.0)then
  print'(a)',prompt//':'
  print'(a,i0,a)','Enter / for default=',iDefault,')'
  read*,num
  write(1,'(i17,6x,a)')num,prompt
else
  print'(a,$)',prompt//':'              ! Print prompt first, in case of i/o error.
  read(1,*,end=99)num
  print'(i0)',num
endif
return
99 print*,'*****Parameter file too short - continue editing from keyboard!*****'
io=0
open(1,access='append')
goto 1
end
! --------------------------------------------------------------------------------------
subroutine WriteBlank                   ! Prints blank line on screen, parameter 
                                        ! file (if io.eq.0), and postscript.
common xmin,ymin,xmax,ymax,x,y,WidLin,io,iExtra
print*,' '
if(io.eq.0)write(1,*)' '
write(2,*)' '
return
end
! --------------------------------------------------------------------------------------

subroutine Readflt(Prompt,val,Default)   ! Similar for float point
common xmin,ymin,xmax,ymax,x,y,WidLin,io,iExtra
character*(*) Prompt
1 val=Default
if(io.eq.0)then
  print'(a)',prompt//':'
  print'(a,g0,a)','(Enter / for default=',Default,')'
  val=Default
  read*,val
  write(1,'(g21.9,2x,a)')val,prompt
else
  print'(a,$)',prompt//':'
  read(1,*,end=99)val
  print'(g0)',val
endif
return
99 print*,'*****Parameter file too short - continue editing from keyboard!*****'
io=0
open(1,access='append')
goto 1
end
! --------------------------------------------------------------------------------------
subroutine MinMaxEndxy(xx,yy)
! Revise xmin,ymin,xmax,ymax,x,y   to include (xx,yy)
common xmin,ymin,xmax,ymax,x,y,WidLin,io,iExtra
xmin=min(xmin,xx)
ymin=min(ymin,yy)
xmax=max(xmin,xx)
ymax=max(ymax,yy)
x=xx
y=yy
!print*,'MinMaxEndxy ',xx,yy,' xymin=',xmin,ymin,' xymax=',xmax,ymax !##
return
end
! --------------------------------------------------------------------------------------
subroutine MoveTo(xx,yy)                  ! Move to xx,yy, to path.
write(2,'(g0,1x,g0,a)')xx,yy,' moveto'
call MinMaxEndxy(xx,yy)                   ! Revise xmin,ymin,xmax,ymax,x,y.
return
end
! --------------------------------------------------------------------------------------
subroutine LineTo(xx,yy)                  ! Similar, but draw line
common xmin,ymin,xmax,ymax,x,y,WidLin,io,iExtra
write(2,'(g0,1x,g0,a)')xx,yy,' lineto'
call MinMaxEndxy(xx,yy)                   
return
end
! --------------------------------------------------------------------------------------
                                          ! Draw CCW circular arc to path.
subroutine DrawArc(xCenter,yCenter,R,theta1,theta2)
common xmin,ymin,xmax,ymax,x,y,WidLin,io,iExtra
!       Input parameters:
! xCenter,yCenter=center of circle
! R=radius of circle
! theta1,theta2=Angle range. Theta1.lt.theta2, and both must be in [-180 to 0] deg.
! xmin,ymin,xmax,ymax = min, max x & y (from previous values)
!       Modified parameters:
! xmin,ymin,xmax,ymax = min, max x & y (including these values)
! x,y=ending x,y
write(2,'(5(g0,1x),a)')xCenter,yCenter,R,theta1,theta2,'arc'
 call MinMaxEndxy(xCenter+R*cosd(theta1),yCenter+R*sind(theta1))

if(iExtra.ne.0)then                       ! Extra: draw radius lines
  write(2,'(a)')'% ***Next 3 lines are radius lines, not part of blade***'
  write(2,'(g0,1x,g0,a)')xCenter+R*cosd(theta1),yCenter+R*sind(theta1),' moveto'
  write(2,'(g0,1x,g0,a)')xCenter,yCenter,' lineto'
  write(2,'(g0,1x,g0,a)')xCenter+R*cosd(theta2),yCenter+R*sind(theta2),' lineto'
endif
                                          ! Include bottom of circle in min,max
if(theta1.lt.-90.and.theta2.gt.-90) &
&call MinMaxEndxy(xCenter+R*cosd(-90.  ),yCenter+R*sind(-90.  ))
 call MinMaxEndxy(xCenter+R*cosd(theta2),yCenter+R*sind(theta2))
return
end
! --------------------------------------------------------------------------------------
subroutine DrawSeg(x1,y1,x2,y2)           ! Draw line segment outside path, 
                                          ! but don't change min,max x,y values
write(2,'(a,g0,1x,g0,a,g0,1x,g0,a)') &
 & 'newpath ',x1,y1,' moveto ',x2,y2,' lineto stroke'
end
! --------------------------------------------------------------------------------------
                                          ! Setup rotation, and translation, 
                                          ! so everything fits.
subroutine SetGeometry(TheMargin,xmin2,xmax2,ymin2,ymax2,xPap,yPap, &
 & irotate,scale)

real R(2,2)                               ! Rotation matrix
real x1(2), x2(2), x3(2), x4(2)           ! Corners of zone box x4 x3
                                          !                     x1 x2
real xp1(2),xp2(2),xp3(2),xp4(2)          ! Same after rotation.

!print*,'SetGeometry Margin=',TheMargin,' xmin2max2=',xmin2,xmax2, &
! &' ymin2max2=',ymin2,ymax2,' xyPap=',xPap,yPap, &
! & 'irotate=',irotate,'scale=',scale     ! ##

x1=(/xmin2,ymin2/)                        ! Corners of coordinate range box.
x2=(/xmax2,ymin2/)                        ! With tic borders.
x3=(/xmax2,ymax2/)
x4=(/xmin2,ymax2/)

!print*,'x1=',x1 !##
!print*,'x2=',x2 !##
!print*,'x3=',x3 !##
!print*,'x4=',x4 !##
if(irotate.gt.90.or.rotate.lt.-1)then
  stop 'Rotation angle must be between 0 and 90!'
elseif(irotate.ge.0.and.irotate.le.90)then
  i1=irotate
  i2=irotate
elseif(irotate.eq.-1)then
  i1=0
  i2=90
else
  stop 'Invalid rotation angle!'
endif

do i=i1,i2                                ! Search through rotations.
  rotate=i
  if(rotate.eq.0)then                     ! Exact 0 or 90 degree rotation matrix exactly.
    R=reshape( (/1,0,0,1/),(/2,2/) )
  elseif(rotate.eq.90)then
    R=reshape( (/0,1,-1,0/),(/2,2/) )
  else                                    ! Otherwise calculate rotation matrix
    R=reshape( (/cosd(rotate),sind(rotate),-sind(rotate),cosd(rotate)/),(/2,2/) )
  endif
!print*,'R=        ',R
!print*,'calculated',reshape( (/cosd(rotate),sind(rotate),-sind(rotate),cosd(rotate)/),(/2,2/) )

                                          ! Calculate rotated corners
  xp1=scale*matmul(R,x1)                  ! Inefficient - I don't use them all.
  xp2=scale*matmul(R,x2)                  ! I don't care. 
  xp3=scale*matmul(R,x3)                  ! "p" means prime, i.e., rotated.
  xp4=scale*matmul(R,x4)

!print*,'xp1=',xp1 !##
!print*,'xp2=',xp2 !##
!print*,'xp3=',xp3 !##
!print*,'xp4=',xp4 !##
!print*,' ' !##

                                          ! Rotated box geometry for 0<=rotate<=90
                                          !           xp3
                                          !  xp4      
                                          !             xp2
                                          !    xp1
  xpmin2=xp4(1)                           ! Minimum and maximum rotated limits.
  xpmax2=xp2(1)
  ypmin2=xp1(2)
  ypmax2=xp3(2)
  !print*,'xpminmax2=',xpmin2,xpmax2,'ypminmax2=',ypmin2,ypmax2 !##
                                          ! Does it fit on the page?
  if(xpmax2-xpmin2.le.xPap-2*TheMargin.and. &
   & ypmax2-ypmin2.le.yPap-2*TheMargin) goto 10
enddo
stop 'Sorry! Does not fit on requested page size.'

10 print   '(a,g0,a)','Rotation angle=',rotate,' works!'
                                          ! Final coordinate transformation will be
                                          ! scale*(Xp matmul + A); I now find A)
print*,'Left alignment seems intuitive, maybe can print on small width paper.'
print*,'But right alignment may show more of radius lines.'
iDefault=2
call ReadInt('1=Left Align,2=Right Align)',i,iDefault)

if(i.eq.1)then                            ! scale*(xpmin2+Ax)=TheMargin
  Ax=TheMargin/scale-xpmin2
! print*,'Ax=',TheMargin,'/',scale,'-',xpmin2,'=',Ax !##
else                                      ! scale*(xpmax2+Ax)=xPap-TheMargin
  Ax=(xPap-TheMargin)/scale-xpmax2
! print*,'Ax=','(',xPap,'-',TheMargin,')/',scale,'-',xpmax2,'=',Ax !##
endif
!print*,'Ax=',Ax  !##

Ay=TheMargin/scale-ypmin2                 ! Always use Bottom alignment.
                                          ! scale*(ypmin2+Ay)=TheMargin
! print*,'Ay=',TheMargin,'/',scale,'-',ypmin2,'=',Ay
if(Ax.ne.0.or.Ay.ne.0)write(2,'(g0,1x,g0,a)') Ax,Ay,' translate'
                                          ! I thought that rotate should come 
                                          ! before  translate, but that doesn't 
                                          ! seem to work. This does. Which means
                                          ! there is something about Postscript
                                          ! that I don't understand.
write(2,'(a,g0)')'% Rotation angle=',rotate
if(rotate.ne.0)write(2,'(g0,a)')rotate,' rotate'
return
end
